from matplotlib import pyplot as plt
from matplotlib.patches import Rectangle
from gene import Gene
from transcript import Transcript

def draw_coding_sequence(id, x, currentAxis):
    d = 0.075
    for i in range(int(len(x)/2)):
        x1 = x[2*i]
        x2 = x[2*i+1]
        currentAxis.add_patch(Rectangle((x1 - d, id - d), x2-x1, 2*d,alpha=1,facecolor='grey'))
    return currentAxis

def draw_protein_sequence(id, x, color, currentAxis):
    d = 0.2
    for i in range(int(len(x)/2)):
        x1 = x[2*i]
        x2 = x[2*i+1]
        currentAxis.add_patch(Rectangle((x1 - d, id - d), x2-x1, 2*d,alpha=1,facecolor=color))
    return currentAxis

def draw_gene(gene):
    transcripts = gene.transcripts
    n = len(transcripts)
    fig,ax = plt.subplots()
    currentAxis = plt.gca()
    plt.title(' / '.join(gene.gene_name))
    plt.autoscale()
    plt.ylim(0, n + 1)
    plt.figure()
    color_ids = []
    colors = ['slateblue',  'firebrick', 'springgreen', 'goldenrod', 'palevioletred']
    i = 1
    for t in transcripts:
        if t.protein_id not in color_ids:
            color_ids.append(t.protein_id)
    for c_id in color_ids:
        for t in transcripts:
            if t.protein_id == c_id:
                p_id = color_ids.index(t.protein_id) % len(colors)
                currentAxis = draw_coding_sequence(i, t.chrom_loc, currentAxis)
                currentAxis = draw_protein_sequence(i, t.CDS_loc, colors[p_id], currentAxis)
                i += 1
    plt.rcParams['figure.figsize'] = [10, 5]
    plt.show()



import pandas as pd
import numpy as np
from gene import Gene
from transcript import Transcript

def create_genes(dataset, transcript_id_dataset):
    list_of_genes = []
    genes = np.unique(test_dataset['gene_id'])
    for gene_id in genes:
        gene_dataset = dataset[dataset['gene_id'] == gene_id]
        gene_name = np.unique(transcript_id_dataset[transcript_id_dataset['gene_id'] == gene_id]['gene_name'])
        gene = Gene(gene_id, gene_name)
        transcripts = np.unique(gene_dataset['transcript_id'])
        for transcript_id in transcripts:
            transcript_dataset = gene_dataset[gene_dataset['transcript_id'] == transcript_id]
            chrom_loc = transcript_dataset[transcript_dataset['data_type'] == 'exon'][['x1','x2']].values
            if (transcript_id_dataset[transcript_id_dataset['transcript_id'] == transcript_id]['direction'].tolist()[0] == '-'):
                chrom_loc = np.flipud(chrom_loc)
            chrom_loc = chrom_loc.flatten().astype(int)
            protein_id = transcript_id_dataset[transcript_id_dataset['transcript_id'] == transcript_id]['protein_id'].values[0]
            CDS = transcript_dataset[(transcript_dataset['data_type'] == 'CDS') & (transcript_dataset['source_id'] == 'tappAS')]
            CDS = CDS[['x1','x2']].values[0].astype(int)
            transcript = Transcript(transcript_id, chrom_loc, protein_id, CDS)
            transcript.calculate_relative_loc()
            transcript.calculate_CDS_loc()
            gene.add_transcript(transcript)
        list_of_genes.append(gene)
    return list_of_genes



import os
import pickle

def save_gene_object(obj, gene_name, path='GeneObjects'):
    if not os.path.exists(path):
       os.makedirs(path)
    with open(path+'/'+gene_name+'.pkl', 'wb') as outp:
        pickle.dump(obj, outp, pickle.HIGHEST_PROTOCOL)



class Gene:
    def __init__(self, gene_id, gene_name):
        self.gene_id = gene_id
        self.gene_name = gene_name
        self.transcripts = []
    
    def add_transcript(self, transcript):
        self.transcripts.append(transcript)



from setuptools import setup, find_packages

VERSION = '0.0.1' 
DESCRIPTION = 'Protein Isoform Event identifier'
LONG_DESCRIPTION = 'My first Python package with a slightly longer description'

setup(
        name="isoformevent", 
        version=VERSION,
        author="Julian Perez",
        author_email="<julian.perez@stanford.edu>",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=['pandas', 'numpy', 'os', 'pickle', 'matplotlib'],
        
        keywords=['python', 'protein', 'isoform', 'event'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 2",
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
        ]
)


from savegene import save_gene_object
from readgene import read_gene_object

from gene import Gene
from transcript import Transcript

from creategenes import create_genes

from drawgene import draw_gene




class Transcript:
    def __init__(self, trancript_id, chrom_loc, protein_id, CDS):
        self.trancript_id = trancript_id
        self.chrom_loc = chrom_loc
        self.protein_id = protein_id
        self.relative_loc = []
        self.CDS = CDS
        self.CDS_loc = []
    
    def calculate_relative_loc(self):
        x = self.chrom_loc
        y = []
        delta = x[1] - x[0]
        y = [1, 1 + delta]
        for i in range(1,int(len(x)/2)):
            delta = x[2*i+1] - x[2*i] 
            y.append(y[2*i-1]+1)
            y.append(y[2*i] + delta)
        self.relative_loc = y

    def calculate_CDS_loc(self):
        x = copy.deepcopy(self.chrom_loc)
        y = self.relative_loc
        cds = self.CDS
        i_a = 1
        i_b = len(y) - 2
        delta_a = 0
        delta_b = 0
        for k in range(len(y)):
            if (cds[0] <= y[i_a]):
                i_a -= 1
                delta_a = cds[0] - y[i_a]
                break
            i_a += 1
        for k in range(len(y)):
            if (cds[1] >= y[i_b]):
                i_b += 1
                delta_b = cds[1] - y[i_b]
                break
            i_b -= 1
        z = x[i_a:i_b+1]
        z[0] = z[0] + delta_a
        z[-1] = z[-1] + delta_b
        self.CDS_loc = z
